require "highline/import"
require "json"

module PackageCloud
  class ConfigFile
    attr_reader :token

    def initialize(filename = "~/.packagecloud", url = "packagecloud.io")
      @filename = File.expand_path(filename)
      @url = url
    end

    def read_or_create
      if File.exist?(@filename)
        attrs = JSON.parse(File.read(@filename))
        @token = attrs["token"] if attrs.has_key?("token")
        @url   = attrs["url"] if attrs.has_key?("url")
      else
        puts "No config file exists at #{@filename}. Login to create one."

        @token = login_from_console
        write
      end
    end

    def url
      @url ||= "packagecloud.io"
    end

    def base_url(username = token, password = "")
      "http://#{CGI.escape(username)}:#{CGI.escape(password)}@#{url}"
    end

    private
      def login_from_console
        e     = ask("Email:")
        p     = ask("Password:") { |q| q.echo = false }

        begin
          PackageCloud::Auth.get_token(base_url(e, p))
        rescue RestClient::Unauthorized => e
          puts "Sorry, but we couldn't find you. Give it another try."
          login_from_console
        end
      end

      def write
        print "Got your token. Writing a config file to #{@filename}... "
        attrs = {url: @url, token: @token}
        File.open(@filename, "w", 0600) { |f| f << JSON.dump(attrs); f << "\r\n" }
        puts "success!"
      end
  end
end
